local vector = require "vector"
local scaled = require "scaled"
local sign = math.sign or function(x) return x < 0 and -1 or x > 0 and 1 or 0 end

local fire = {}
fire.image = love.graphics.newImage( "img/800x600/ball.png" )
local x_tile_pos = 0
local y_tile_pos = 19
local tile_width = 8
local tile_height = 18
fire.width = 8
fire.height = 18
fire.quad = love.graphics.newQuad( x_tile_pos, y_tile_pos,
                tile_width, tile_height,
                fire.image )

local ball_wall_sound = love.audio.newSource(
   "sounds/ball_wall/pumpkin_break_01_short_norm.ogg",
   "static")

fire.current_fire = {}

function fire.new_fire( position, speed)
   return( { position = position,
      speed = speed,
      width = fire.width,
      collision_counter = 0,
      quad = fire.quad } )
end

function fire.add_fire( single_fire )
   table.insert( fire.current_fire, single_fire )
end

function fire.update_fire( single_fire, dt )
   single_fire.position.y = single_fire.position.y - single_fire.speed * dt
end

function fire.update( dt )
   for _, single_fire in pairs( fire.current_fire ) do
      fire.update_fire( single_fire, dt )
   end
end

function fire.draw_fire( single_fire )
   scaled.graphics.draw( fire.image,
                single_fire.quad,
                single_fire.position.x,
                single_fire.position.y )
end

function fire.draw()
   for _, single_fire in pairs( fire.current_fire ) do
      fire.draw_fire( single_fire )
   end
end

function fire.reset()
   for i in pairs( fire.current_fire ) do
      fire.current_fire[i] = nil
   end
end

return fire
