gamestates = require "gamestates"
scaled = require "scaled"
local qrencode = require "lib/qrencode"

music = love.audio.newSource( "sounds/music/S31-Night Prowler.ogg", "static" )
music:setLooping( true )
credits = 0

function isModuleAvailable(name)
    if package.loaded[name] then
        return true
    else
        for _, searcher in ipairs(package.searchers or package.loaders) do
            local loader = searcher(name)
            if type(loader) == 'function' then
                package.preload[name] = loader
                return true
            end
        end
        return false
    end
end

USING_ARCADE = false
for i, cmd in ipairs(arg) do
    if cmd == "--arcade" then
        USING_ARCADE = true
    end
end
if not isModuleAvailable("luaarcade") then
    USING_ARCADE = false
end
if USING_ARCADE then
   Arcade = require "luaarcade"
   arcade_info = love.filesystem.getInfo("arcade.json")
   if arcade_info ~= nil then
      file = love.filesystem.newFile("arcade.json")
      file:open("r")
      arcade_json = file:read()
      payLink = Arcade.createLink(arcade_json)
   end
end

function love.load()
   scaled.change(1, true);
   if USING_ARCADE then
      local ok, tab = qrencode.qrcode(payLink)
      if ok then
         local qrPayLinkData = love.image.newImageData(#tab * 20 + 80, #tab * 20 + 80)
         for x = 1, #tab * 20 + 80 do
            for y = 1, #tab * 20 + 80 do
               qrPayLinkData:setPixel(x - 1, y - 1, 1, 1, 1, 1)
            end
         end
         for x = 1, #tab do
            for y = 1, #tab do
               for yOffset = 0, 19 do
                  for xOffset = 0, 19 do
                     if tab[x][y] > 0 then
                        qrPayLinkData:setPixel((x - 1) * 20 + xOffset + 40, (y - 1) * 20 + yOffset + 40, 0, 0, 0, 1)
                     else
                        qrPayLinkData:setPixel((x - 1) * 20 + xOffset + 40, (y - 1) * 20 + yOffset + 40, 1, 1, 1, 1)
                     end
                  end
               end
            end
         end
         qrPayLink = love.graphics.newImage(qrPayLinkData)
      end
   end
   gamestates.set_state( "menu" )
end

function love.update( dt )
   gamestates.state_event( "update", dt )
   if USING_ARCADE then
      credits = credits + Arcade.getCredit()
   end
end

function love.draw()
   gamestates.state_event( "draw" )
end

function love.keyreleased( key, code )
   gamestates.state_event( "keyreleased", key, code )
end

function love.keypressed( key, code )
   gamestates.state_event( "keypressed", key, code )
end

function love.mousereleased( x, y, button, istouch )
   gamestates.state_event( "mousereleased", x, y, button, istouch )
end

function love.quit()
   if USING_ARCADE then
      Arcade.quit()
   end
end
