local vector = require "vector"
local buttons = require "buttons"

local menu = {}
local bungee_font = love.graphics.newFont(
   "/fonts/Bungee_Inline/BungeeInline-Regular.ttf", 30 )

local menu_buttons_image = love.graphics.newImage( "img/800x600/buttons.png" )
local button_tile_width = 128
local button_tile_height = 64
local play_button_tile_x_pos = 0
local play_button_tile_y_pos = 0
local quit_button_tile_x_pos = 0
local quit_button_tile_y_pos = 64
local play2_button_tile_x_pos = 0
local play2_button_tile_y_pos = 128
local selected_x_shift = 128
local play_button_quad = love.graphics.newQuad(
   play_button_tile_x_pos,
   play_button_tile_y_pos,
   button_tile_width,
   button_tile_height,
   menu_buttons_image )
local play_button_selected_quad = love.graphics.newQuad(
   play_button_tile_x_pos + selected_x_shift,
   play_button_tile_y_pos,
   button_tile_width,
   button_tile_height,
   menu_buttons_image )
local quit_button_quad = love.graphics.newQuad(
   quit_button_tile_x_pos,
   quit_button_tile_y_pos,
   button_tile_width,
   button_tile_height,
   menu_buttons_image )
local quit_button_selected_quad = love.graphics.newQuad(
   quit_button_tile_x_pos + selected_x_shift,
   quit_button_tile_y_pos,
   button_tile_width,
   button_tile_height,
   menu_buttons_image )
local play2_button_quad = love.graphics.newQuad(
   play2_button_tile_x_pos,
   play2_button_tile_y_pos,
   button_tile_width,
   button_tile_height,
   menu_buttons_image )
local play2_button_selected_quad = love.graphics.newQuad(
   play2_button_tile_x_pos + selected_x_shift,
   play2_button_tile_y_pos,
   button_tile_width,
   button_tile_height,
   menu_buttons_image )

local start_button = {}
local start2_button = {}
local quit_button = {}

function menu.load( prev_state, ... )
   if qrPayLink then
      qr_pay_link_quad = love.graphics.newQuad(0, 0, qrPayLink:getWidth(), qrPayLink:getHeight(), qrPayLink)
   end
   start_button = buttons.new_button{
      text = "New game",
      position = vector( (1056 - button_tile_width) / 2, 200),
      width = button_tile_width,
      height = button_tile_height,
      image = menu_buttons_image,
      quad = play_button_quad,
      quad_when_selected = play_button_selected_quad
   }
   start2_button = buttons.new_button{
      text = "New 2 player game",
      position = vector( (1056 - button_tile_width) / 2, 310),
      width = button_tile_width,
      height = button_tile_height,
      image = menu_buttons_image,
      quad = play2_button_quad,
      quad_when_selected = play2_button_selected_quad
   }
   quit_button = buttons.new_button{
      text = "Quit",
      position = vector( (1056 - button_tile_width) / 2, 420),
      width = button_tile_width,
      height = button_tile_height,
      image = menu_buttons_image,
      quad = quit_button_quad,
      quad_when_selected = quit_button_selected_quad
   }
   music:play()
end

function menu.update( dt )
   buttons.update_button( start_button, dt )
   buttons.update_button( start2_button, dt )
   buttons.update_button( quit_button, dt )
end

function menu.draw()
   if qrPayLink then
      love.graphics.draw(qrPayLink, qr_pay_link_quad, 0, 0, 0, 1, 1);
   end
   if USING_ARCADE then
      local oldfont = love.graphics.getFont()
      love.graphics.setFont( bungee_font )
      local r, g, b, a = love.graphics.getColor( )
      love.graphics.setColor( 1, 1, 1, 0.9 )
      scaled.graphics.printf( "Credits:",
             430,
             550,
             200,
             "left" )
      scaled.graphics.printf( credits,
             600,
             550,
             200,
             "left" )
      love.graphics.setFont( oldfont )
      love.graphics.setColor( r, g, b, a )
   end
   buttons.draw_button( start_button )
   buttons.draw_button( start2_button )
   buttons.draw_button( quit_button )
end

function menu.keyreleased( key, code )
   if key == "return" then
      gamestates.set_state( "game", { current_level = 1 } )
   elseif key == 'escape' then
      love.event.quit()
   elseif key == '1' then
      if USING_ARCADE then
         if credits > 0 then
            credits = credits - 1
            gamestates.set_state( "game", { current_level = 1, players = 1 } )
         end
      else
         gamestates.set_state( "game", { current_level = 1, players = 1 } )
      end
   elseif key == '2' then
      if USING_ARCADE then
         if credits > 0 then
            credits = credits - 1
            gamestates.set_state( "game", { current_level = 1, players = 2 } )
         end
      else
         gamestates.set_state( "game", { current_level = 1, players = 2 } )
      end
   end    
end

function menu.mousereleased( x, y, button, istouch )
   if button == 'l' or button == 1 then
      if buttons.mousereleased( start_button, x, y, button ) then
         if USING_ARCADE then
            if credits > 0 then
               credits = credits - 1
               gamestates.set_state( "game", { current_level = 1, players = 1 } )
            end
         else
            gamestates.set_state( "game", { current_level = 1, players = 1 } )
         end
      elseif buttons.mousereleased( start2_button, x, y, button ) then
      if USING_ARCADE then
         if credits > 0 then
            credits = credits - 1
            gamestates.set_state( "game", { current_level = 1, players = 2 } )
         end
      else
         gamestates.set_state( "game", { current_level = 1, players = 2 } )
      end
      elseif buttons.mousereleased( quit_button, x, y, button ) then
         love.event.quit()
      end
   elseif button == 'r' or button == 2 then
      love.event.quit()
   end    
end

return menu
