local vector = require "vector"
local scaled = require "scaled"
local fire = require "fire"
local balls = require "balls"

platform = Class {}

function platform:init(player, control)
   self.position = vector( 300, 560 )
   self.speed = vector( 800, 0 )
   self.image = love.graphics.newImage( "img/800x600/platform.png" )
   self.small_tile_width = 75
   self.small_tile_height = 16
   self.small_tile_x_pos = 0
   self.small_tile_y_pos = 0 + 96 * player
   self.norm_tile_width = 108
   self.norm_tile_height = 16
   self.norm_tile_x_pos = 0
   self.norm_tile_y_pos = 32 + 96 * player
   self.large_tile_width = 141
   self.large_tile_height = 16
   self.large_tile_x_pos = 0
   self.large_tile_y_pos = 64 + 96 * player
   self.glued_x_pos_shift = 192
   self.quad = love.graphics.newQuad( self.norm_tile_x_pos,
                      self.norm_tile_y_pos,
                      self.norm_tile_width,
                      self.norm_tile_height,
                      self.image )
   self.gun_quad = love.graphics.newQuad( 0,
                      193,
                      26,
                      24,
                      self.image )
   self.width = self.norm_tile_width
   self.height = self.norm_tile_height
   self.size = "norm"
   self.glued = false
   self.activated_next_level_bonus = false
   self.speed = 600
   self.fire_speed = 600
   self.controller = control
   self.fire = 0
end

function platform:update( dt )
   if self.controller == nil then
      self:follow_mouse( dt )
   else
      self:move(dt)
   end
   local try_fire = self.controller:fire_pressed()
   if try_fire then
      balls.launch_single_ball_from_platform(self)
   end
   if self.fire > 0 then
      if try_fire then
         local position = self.position:clone()
         position.x = position.x + self.width / 2
         position.y = position.y - 10;
         fire.add_fire( fire.new_fire(
                  position, self.fire_speed) )
      end
      self.fire = self.fire - dt
      if self.fire < 0 then
         self.fire = 0
      end
   end
end

function platform:draw()
   scaled.graphics.draw( self.image,
                self.quad,
                self.position.x,
                self.position.y )
   if self.fire > 0 then
      scaled.graphics.draw( self.image,
                   self.gun_quad, 
                   self.position.x + self.width / 2 - 13,
                   self.position.y - 9 )
   end
end

function platform:move( dt )
   local left_wall = 34
   local right_wall_minus_platform = 832 - self.width
   if self.controller:left_pressed() then
      self.position.x = self.position.x - self.speed * dt
      if self.position.x < left_wall then
         self.position.x = left_wall - self.width / 2
      end
   end
   if self.controller:right_pressed() then
      self.position.x = self.position.x + self.speed * dt
      if self.position.x > right_wall_minus_platform then
         self.position.x = right_wall_minus_platform
      end
   end
end

function platform:follow_mouse( dt )
   local x, y = love.mouse.getPosition()
   local left_wall_plus_half_platform = 34 + self.width / 2
   local right_wall_minus_half_platform = 832 - self.width / 2
   if ( x > left_wall_plus_half_platform and
   x < right_wall_minus_half_platform ) then
      self.position.x = x - self.width / 2
   elseif x < left_wall_plus_half_platform then
      self.position.x =
    left_wall_plus_half_platform - self.width / 2
   elseif x > right_wall_minus_half_platform then
      self.position.x =
    right_wall_minus_half_platform - self.width / 2
   end
end

function platform:bounce_from_wall( shift_platform, wall )
   self.position.x = self.position.x + shift_platform.x
   if wall.next_level_bonus then
      self.activated_next_level_bonus = true
   end
end

function platform:react_on_decrease_bonus()
   if self.size == "norm" then
      self.width = self.small_tile_width
      self.height = self.small_tile_height
      self.quad = love.graphics.newQuad(
         self.small_tile_x_pos, self.small_tile_y_pos,
         self.small_tile_width, self.small_tile_height,
         self.image )
      self.size = "small"
   elseif self.size == "large" then
      self.width = self.norm_tile_width
      self.height = self.norm_tile_height
      self.quad = love.graphics.newQuad(
         self.norm_tile_x_pos, self.norm_tile_y_pos,
         self.norm_tile_width, self.norm_tile_height,
         self.image )
      self.size = "norm"   
   end
end

function platform:react_on_increase_bonus()
   if self.size == "small" then
      self.width = self.norm_tile_width
      self.height = self.norm_tile_height
      self.quad = love.graphics.newQuad(
         self.norm_tile_x_pos, self.norm_tile_y_pos,
         self.norm_tile_width, self.norm_tile_height,
         self.image )
      self.size = "norm"
   elseif self.size == "norm" then
      self.width = self.large_tile_width
      self.height = self.large_tile_height
      self.quad = love.graphics.newQuad(
         self.large_tile_x_pos, self.large_tile_y_pos,
         self.large_tile_width, self.large_tile_height,
         self.image )
      self.size = "large"
   end
end

function platform:reset_size_to_norm()
   self.width = self.norm_tile_width
   self.height = self.norm_tile_height
   self.quad = love.graphics.newQuad(
      self.norm_tile_x_pos, self.norm_tile_y_pos,
      self.norm_tile_width, self.norm_tile_height,
      self.image )
   self.size = "norm"   
end


function platform:react_on_glue_bonus()
   self.glued = true
   if self.size == "small" then
      self.quad = love.graphics.newQuad(
         self.small_tile_x_pos + self.glued_x_pos_shift,
         self.small_tile_y_pos,
         self.small_tile_width,
         self.small_tile_height,
         self.image )
   elseif self.size == "norm" then
      self.quad = love.graphics.newQuad(
         self.norm_tile_x_pos + self.glued_x_pos_shift,
         self.norm_tile_y_pos,
         self.norm_tile_width,
         self.norm_tile_height,
         self.image )
   elseif self.size == "large" then
      self.quad = love.graphics.newQuad(
         self.large_tile_x_pos + self.glued_x_pos_shift,
         self.large_tile_y_pos,
         self.large_tile_width,
         self.large_tile_height,
         self.image )
   end   
end

function platform:remove_glued_effect()
   if self.glued then
      self.glued = false
      if self.size == "small" then
    self.quad = love.graphics.newQuad(
       self.small_tile_x_pos,
       self.small_tile_y_pos,
       self.small_tile_width,
       self.small_tile_height,
       self.image )
      elseif self.size == "norm" then
    self.quad = love.graphics.newQuad(
       self.norm_tile_x_pos,
       self.norm_tile_y_pos,
       self.norm_tile_width,
       self.norm_tile_height,
       self.image )
      elseif self.size == "large" then
    self.quad = love.graphics.newQuad(
       self.large_tile_x_pos,
       self.large_tile_y_pos,
       self.large_tile_width,
       self.large_tile_height,
       self.image )
      end
   end
end

function platform:react_on_fire_bonus()
   self.fire = 10
end

function platform:remove_fire_effect()
   self.fire = 0
end

function platform:remove_bonuses_effects()
   self:remove_glued_effect()
   self:remove_fire_effect()
   self:reset_size_to_norm()
   self.activated_next_level_bonus = false
end
