local vector = require "vector"
local scaled = require "scaled"

local walls = {}

local image = love.graphics.newImage( "img/800x600/walls.png" )
local wall_vertical_tile_width = 32
local wall_vertical_tile_height = 96
local wall_vertical_tile_x_pos = 0
local wall_vertical_tile_y_pos = 0
local wall_horizontal_tile_width = 96
local wall_horizontal_tile_height = 26
local wall_horizontal_tile_x_pos = 64
local wall_horizontal_tile_y_pos = 0
local topleft_corner_tile_width = 64
local topleft_corner_tile_height = 64
local topleft_corner_tile_x_pos = 64
local topleft_corner_tile_y_pos = 32
local topright_corner_tile_width = 64
local topright_corner_tile_height = 64
local topright_corner_tile_x_pos = 128
local topright_corner_tile_y_pos = 32
local center_horizontal_tile_width = 32
local center_horizontal_tile_height = 26
local center_horizontal_tile_x_pos = 0
local center_horizontal_tile_y_pos = 120
local vertical_quad = love.graphics.newQuad(
   wall_vertical_tile_x_pos,
   wall_vertical_tile_y_pos,
   wall_vertical_tile_width,
   wall_vertical_tile_height,
   image )
local horizontal_quad = love.graphics.newQuad(
   wall_horizontal_tile_x_pos,
   wall_horizontal_tile_y_pos,
   wall_horizontal_tile_width,
   wall_horizontal_tile_height,
   image )
local topright_corner_quad = love.graphics.newQuad(
   topright_corner_tile_x_pos,
   topright_corner_tile_y_pos,
   topright_corner_tile_width,
   topright_corner_tile_height,
   image )
local topleft_corner_quad = love.graphics.newQuad(
   topleft_corner_tile_x_pos,
   topleft_corner_tile_y_pos,
   topleft_corner_tile_width,
   topleft_corner_tile_height,
   image )
local center_quad = love.graphics.newQuad(
   center_horizontal_tile_x_pos,
   center_horizontal_tile_y_pos,
   center_horizontal_tile_width,
   center_horizontal_tile_height,
   image )

walls.side_walls_thickness = 34
walls.top_wall_thickness = 26
walls.right_border_x_pos = 832
walls.current_level_walls = {}


function walls.new_wall( position, width, height, layout )
   return( { position = position,
        width = width,
        height = height,
        layout = layout } )
end

function walls.update_wall( single_wall )
end

function walls.draw_wall( single_wall )
   if ( single_wall.layout == 'top' ) then
      scaled.graphics.draw(
         image, topleft_corner_quad,
         single_wall.position.x, single_wall.position.y )
      scaled.graphics.draw(
         image, topright_corner_quad,
         single_wall.position.x + single_wall.width -
            topleft_corner_tile_width / 2,
         single_wall.position.y )
      local repeat_n_times = 2
      for i = 0, repeat_n_times do
         shift_x = topleft_corner_tile_width +
            i * wall_horizontal_tile_width
         scaled.graphics.draw(
            image, horizontal_quad,
            single_wall.position.x + shift_x, single_wall.position.y )
      end
      shift_x = topleft_corner_tile_width +
         3 * wall_horizontal_tile_width
      scaled.graphics.draw(
         image, center_quad,
         single_wall.position.x + shift_x, single_wall.position.y )
      shift_x = topleft_corner_tile_width +
         3 * wall_horizontal_tile_width + center_horizontal_tile_width
      scaled.graphics.draw(
         image, horizontal_quad,
         single_wall.position.x + shift_x, single_wall.position.y )
      shift_x = topleft_corner_tile_width +
         4 * wall_horizontal_tile_width + center_horizontal_tile_width
      scaled.graphics.draw(
         image, center_quad,
         single_wall.position.x + shift_x, single_wall.position.y )
      for i = 0, repeat_n_times do
         shift_x = topleft_corner_tile_width +
            (i + 4) * wall_horizontal_tile_width + center_horizontal_tile_width * 2
         scaled.graphics.draw(
            image, horizontal_quad,
            single_wall.position.x + shift_x, single_wall.position.y )
      end
   elseif single_wall.layout == 'left' then
      local repeat_n_times = math.floor(
         (single_wall.height - topright_corner_tile_height) /
            wall_vertical_tile_height )
      for i = 0, repeat_n_times do
         shift_y = topright_corner_tile_height +
            i * wall_vertical_tile_height
         scaled.graphics.draw( image,
            vertical_quad,
            single_wall.position.x,
            single_wall.position.y + shift_y )
      end
   elseif single_wall.layout == 'right' then
      local repeat_n_times = math.floor(
         (single_wall.height - topright_corner_tile_height) /
            wall_vertical_tile_height )
      for i = 0, repeat_n_times do
         if single_wall.next_level_bonus and i == repeat_n_times then
            shift_y = topright_corner_tile_height +
               i * wall_vertical_tile_height + 64
            scaled.graphics.draw( image,
               vertical_quad,
               single_wall.position.x,
               single_wall.position.y + shift_y )
         else
            shift_y = topright_corner_tile_height +
               i * wall_vertical_tile_height
            scaled.graphics.draw( image,
               vertical_quad,
               single_wall.position.x,
               single_wall.position.y + shift_y )
         end
      end
   end
end

function walls.construct_walls()
   local left_wall = walls.new_wall(
      vector( 0, 0 ),
      walls.side_walls_thickness,
      scaled.graphics.getHeight(),
      "left"
   )
   local right_wall = walls.new_wall(
      vector( walls.right_border_x_pos, 0 ),
      walls.side_walls_thickness,
      scaled.graphics.getHeight(),
      "right"
   )
   local top_wall = walls.new_wall(
      vector( 0, 0 ),
      walls.right_border_x_pos,
      walls.top_wall_thickness,
      "top"
   )
   walls.current_level_walls["left"] = left_wall
   walls.current_level_walls["right"] = right_wall
   walls.current_level_walls["top"] = top_wall
end

function walls.update( dt )
end

function walls.draw()
   for _, wall in pairs( walls.current_level_walls ) do
      walls.draw_wall( wall )
   end
end

function walls.remove_bonuses_effects()
   walls.current_level_walls["right"].next_level_bonus = false
end

return walls
